;================================================================
;
; Sound Synth network library
;
; Nmeth Gbor 2007.01.
;
;================================================================

	include common.inc

; Virtual function prototypes
		CONSTRUCTOR		typedef PROTO dwCode:DWORD
		DESTRUCTOR		typedef PROTO 
		BUILD			typedef PROTO lpBinData:DWORD
		BUILDFROMXML		typedef PROTO lpXMLnode:DWORD
		EXPORT			typedef PROTO lpBinData:DWORD
		EXPORTTOXML		typedef PROTO lpXMLnode:DWORD
		CONNECT			typedef PROTO dwGateID:DWORD,lpOutput:DWORD
		DISCONNECT		typedef PROTO dwGateID:DWORD
		GETINPUT		typedef PROTO dwID:DWORD
		GETOUTPUT		typedef PROTO dwID:DWORD
		INFO			typedef PROTO lpBuffer:DWORD
		RESET			typedef PROTO
		RUN			typedef PROTO

		LPCONSTRUCTOR		typedef PTR CONSTRUCTOR
		LPDESTRUCTOR		typedef PTR DESTRUCTOR
		LPBUILD			typedef PTR BUILD
		LPBUILDFROMXML		typedef PTR BUILDFROMXML
		LPEXPORT		typedef PTR EXPORT
		LPEXPORTTOXML		typedef PTR EXPORTTOXML
		LPCONNECT		typedef PTR CONNECT
		LPDISCONNECT		typedef PTR DISCONNECT
		LPGETINPUT		typedef PTR GETINPUT
		LPGETOUTPUT		typedef PTR GETOUTPUT
		LPINFO			typedef PTR INFO
		LPRESET			typedef PTR RESET
		LPRUN			typedef PTR RUN

SETR4		MACRO d:REQ,s:REQ
		push	DWORD PTR [s]
		pop	DWORD PTR [d]
ENDM
		SAMPLE			TEXTEQU <REAL4>
		SMPtoA			TEXTEQU <R4toA>
		SETSAMPLE		TEXTEQU <SETR4>

FCMP		MACRO src:REQ,dst:REQ

		fld	src
		fcomp	dst
		fstsw	ax
		sahf
ENDM


VFT		STRUCT	; Virtual function table

		;Constructor		LPCONSTRUCTOR ?
		Destructor		LPDESTRUCTOR ?
		Build			LPBUILD ?		; build up the module from binary data
		BuildFromXML		LPBUILDFROMXML ?	; build up the module from XML entity
		Export			LPEXPORT ?		; create binary data that describes the module
		ExportToXML		LPEXPORTTOXML ?		; create XML entity that describes the module
		Connect			LPCONNECT ?		; connect an input
		Disconnect		LPDISCONNECT ?		; disconnect an input
		GetInput		LPGETINPUT ?		; get inputs
		GetOutput		LPGETOUTPUT ?		; get outputs
		Info			LPINFO ?		; get infos
		Reset			LPRESET ?		; reset to defaults

VFT		ENDS

		LPVFT	typedef PTR VFT

newObj		MACRO objClass:REQ,objCode
; Macro for  creating new objects dynically
LOCAL string
		string CATSTR <objClass>, <_CLASS>
			%invoke GlobalAlloc,GPTR,size string
			mov	ecx,eax
		string CATSTR <objClass>, <_constructor>
		IFNDEF objCode
			%invoke string,0
		ELSE
			%invoke string,objCode
		ENDIF		
ENDM

@obj		MACRO objName,objClass
; Macro for defining (static) objects in the .data section 
LOCAL objDef,objType

		objDef CATSTR <objName>, <! >, <objClass>, < <>>
		objDef
		objType CATSTR <objName>, <type>, < TEXTEQU >, <!<>, <objClass>, <!>> 
		objType

ENDM

@initObj	MACRO objName:REQ,objCode
; Init object by calling the appropriate constructor
LOCAL objType,objConst

		mov	ecx,offset objName
		objType CATSTR <objName>, <type>
		objConst CATSTR @SubStr (%objType,1,3), <_constructor>
		IFNDEF objCode
			%invoke objConst,0
		ELSE
			%invoke objConst,objCode
		ENDIF
ENDM

_method		MACRO arglist:VARARG
; Macro for calling a method of an object internally (this pointer is already in ecx)
		mov edx,DWORD PTR [ecx]
		%invoke [edx].VFT.arglist
ENDM

@method		MACRO arglist:VARARG
; Macro for calling a method of an object (general case)
; ecx and eax cannot be arguments!
LOCAL txt,arg,count
		push	ecx
		push	edx
		count = 0
%		FOR arg,<arglist>
			IF count EQ 0
				mov	ecx,arg
			ELSEIF count EQ 1
				txt TEXTEQU <arg>
			ELSE
				txt CATSTR txt,<!,>,<arg>
			ENDIF
			count = count + 1
		ENDM
		mov edx,DWORD PTR [ecx]
		%invoke [edx].VFT.txt
		pop	edx
		pop	ecx
ENDM


GATEBINDATA	STRUCT

		mdlID		dw ?
		outputID	db ?
		inputID		db ?

GATEBINDATA	ENDS

		GATE_info	PROTO lpGate:DWORD,lpStr:DWORD,lpszGateLbl:DWORD


NET_MDL_METADATA	STRUCT 
;
; Structure for internal administration
;
			mType	dd ?			; module type
			mSize	dd ?			; size of module
			mAddr	dd ?			; start address of allocated space
			mFlags	dd ?			; administrative and misc. data
NET_MDL_METADATA	ENDS

	MD_FLAG_VISITED	EQU 00000001


; Classes 
; Name   code 
		__SSN_NET_CLID 		EQU	 0000h 
		__SSN_PORT_CLID 	EQU	 0100h 
		__SSN_CONST_CLID 	EQU	 0200h 
		__SSN_POT_CLID 		EQU	 0300h 
		__SSN_OP_CLID 		EQU	 0400h 
		__SSN_OP2_CLID 		EQU	 0500h 
		__SSN_OP4_CLID 		EQU	 0600h 
		__SSN_OP6_CLID 		EQU	 0700h 
		__SSN_OP8_CLID 		EQU	 0800h 
		__SSN_COMP_CLID 	EQU	 0900h 
		__SSN_OSC_CLID 		EQU	 0A00h 
; Module types 
; Name   code 
		_subnet 		EQU	 __SSN_NET_CLID+00h 
		_port 			EQU	 __SSN_PORT_CLID+00h 
		_inport 		EQU	 __SSN_PORT_CLID+01h 
		_outport 		EQU	 __SSN_PORT_CLID+02h 
		_const 			EQU	 __SSN_CONST_CLID+00h 
		_pot 			EQU	 __SSN_POT_CLID+00h 
		_not 			EQU	 __SSN_OP_CLID+00h 
		_neg 			EQU	 __SSN_OP_CLID+01h 
		_div 			EQU	 __SSN_OP_CLID+02h 
		_add2 			EQU	 __SSN_OP2_CLID+00h 
		_mul2 			EQU	 __SSN_OP2_CLID+01h 
		_or2 			EQU	 __SSN_OP2_CLID+02h 
		_and2 			EQU	 __SSN_OP2_CLID+03h 
		_div2 			EQU	 __SSN_OP2_CLID+04h 
		_add4 			EQU	 __SSN_OP4_CLID+00h 
		_mul4 			EQU	 __SSN_OP4_CLID+01h 
		_or4 			EQU	 __SSN_OP4_CLID+02h 
		_and4 			EQU	 __SSN_OP4_CLID+03h 
		_add6 			EQU	 __SSN_OP6_CLID+00h 
		_mul6 			EQU	 __SSN_OP6_CLID+01h 
		_or6 			EQU	 __SSN_OP6_CLID+02h 
		_and6 			EQU	 __SSN_OP6_CLID+03h 
		_add8 			EQU	 __SSN_OP8_CLID+00h 
		_mul8 			EQU	 __SSN_OP8_CLID+01h 
		_or8 			EQU	 __SSN_OP8_CLID+02h 
		_and8 			EQU	 __SSN_OP8_CLID+03h 
		_comp 			EQU	 __SSN_COMP_CLID+00h 
		_imp 			EQU	 __SSN_OSC_CLID+00h 
		_saw 			EQU	 __SSN_OSC_CLID+01h 
		_sin 			EQU	 __SSN_OSC_CLID+02h 
		_noise 			EQU	 __SSN_OSC_CLID+03h 
		_pulse 			EQU	 __SSN_OSC_CLID+04h 
		_psaw 			EQU	 __SSN_OSC_CLID+05h 
		_tri 			EQU	 __SSN_OSC_CLID+06h 

       		OSC_LAST_CODE		EQU	 _noise 

	include inc\mdl.inc
	include inc\net.inc
	include inc\port.inc
	include inc\inp.inc
	include inc\osc.inc

	include masm32.inc
	include debug.inc
	includelib masm32.lib
	includelib debug.lib
